'''Version comparison classes.
'''

from distutils.version import LooseVersion

class Version(object):
	'''Represents a single version.
	'''

	def __init__(self, versionString):
		if versionString is None or not versionString:
			raise RuntimeError("Can't make a Version with None")
		if not isinstance(versionString, str):
			raise RuntimeError("Trying to make a Version with non-string object {0}".format(versionString))
		self.string = versionString
		self.value = LooseVersion(self.string)

	def __repr__(self):
		return "(version {0})".format(self.value.__str__())

	def __eq__(self, other):
		if isinstance(other, Version):
				return self.value == other.value
		else:
				return NotImplemented

	def __ne__(self, other):
		return not self.__eq__(other)

	def __lt__(self, other):
		if isinstance(other, Version):
				return self.value < other.value
		else:
				return NotImplemented

	def __ge__(self, other):
		return not self.__lt__(other)

	def __gt__(self, other):
		if isinstance(other, Version):
				return self.value > other.value
		else:
				return NotImplemented

	def __le__(self, other):
		return not self.__gt__(other)

class VersionPair(object):
	'''Specifies an inclusive range of valid versions.
	A None value for the minimum or maximum indicates that
	that side of the range is unbounded.
	'''

	def __init__(self, minVersionString=None, maxVersionString=None):
		self.minimum = None
		if minVersionString is not None and minVersionString:
			if isinstance(minVersionString, str):
					self.minimum = Version(minVersionString)
			elif isinstance(minVersionString, Version):
					self.minimum = minVersionString
			else:
					raise RuntimeError("Minimum isn't a string or Version object")
		self.maximum = None
		if maxVersionString is not None and maxVersionString:
			if isinstance(maxVersionString, str):
					self.maximum = Version(maxVersionString)
			elif isinstance(maxVersionString, Version):
					self.maximum = maxVersionString
			else:
					raise RuntimeError("Maximum isn't a string or Version object")
		#Make sure range makes sense.
		if self.minimum is not None and self.maximum is not None:
			if self.minimum > self.maximum:
				#TODO: find better error type
				self.minimum = None
				self.maximum = None
				errString = "Trying to make VersionPair with min={0} and max={1}"
				raise RuntimeError(errString.format(minVersionString, maxVersionString))

	def __repr__(self):
		minSet = self.minimum is not None
		maxSet = self.maximum is not None
		minRepr = ""
		if minSet:
			minRepr = "min: {0}".format(self.minimum)
		maxRepr = ""
		if maxSet:
			maxRepr = "max: {0}".format(self.maximum)
		separatorRepr = ""
		if minSet and maxSet:
			separatorRepr = ", "
		return "(version pair, {0}{1}{2})".format(minRepr, separatorRepr, maxRepr)

	def versionIsAcceptable(self, version):
		'''Checks that `version` is in the range
		[minimum, maximum].
		Returns True if `version` is in that range,
		False otherwise.
		'''

		minAcceptable = True
		maxAcceptable = True
		if self.minimum is not None:
			minAcceptable = version >= self.minimum
		if self.maximum is not None:
			maxAcceptable = version <= self.maximum
		return minAcceptable and maxAcceptable



'''Unit tests for context module.
'''

import unittest
import platform
from ...setup.context import Context, Logger

class ContextTest(unittest.TestCase):
	'''Unit tests for context module.
	'''

	def testDefaultValidity(self):
		'''Tests that a default-constructed
		Context is valid on this platform.
		'''
		logger = Logger()
		ctx = Context(logger)
		self.assertTrue(ctx.isValid)

	def testContextDisplay(self):
		'''Tests the two context display methods.
		'''
		logger = Logger()
		ctx = Context(logger)
		print("")
		print("Context full display:")
		ctx.printContextDescription()
		print("Context summary:")
		ctx.printContextSummary()

	def testOSQuery(self):
		'''Tests that Context.osIs*() methods
		return their expected values.
		'''

		#Not really useful, since Context
		#basically uses os.platform directly for this.
		passed = True
		logger = Logger()
		ctx = Context(logger)
		expectedResults = (platform.system() == "Windows",
			platform.system() == "Linux",
			platform.system() == "Darwin")
		methodsToTest = (ctx.osIsWindows, ctx.osIsLinux, ctx.osIsMacOS)
		for i in range(0, len(expectedResults)):
			currExpectedResult = expectedResults[i]
			currMethod = methodsToTest[i]
			actualResult = currMethod()
			if actualResult != currExpectedResult:
				passed = False
				print("Expected {0}() to return {1}, got {2}".format(currMethod.__name__,
					currExpectedResult, actualResult))

		self.assertTrue(passed)

if __name__ == "__main__":
	suite = unittest.TestLoader().loadTestsFromTestCase(ContextTest)
	unittest.TextTestRunner(verbosity=2).run(suite)



'''Integration tests for basis.
'''

import unittest
from .sampleteststeps import TestSteps
from ..basis import Basis
from ..setup import Flags, Logger

#Create some sample steps, run in check only and in install mode.
#Assert that specific steps succeed and fail.
def _runSteps(step):
	step.perform(TestSteps.Pass)
	step.perform(TestSteps.Fail)
	step.perform(TestSteps.Unnecessary)
	step.perform(TestSteps.AlreadyDone)
	step.perform(TestSteps.NotDone)
	step.perform(TestSteps.WrongVersion)

###
# ENTRY POINT
###
class IntegrationTest(unittest.TestCase):
	'''Integration tests for basis.
	'''

	def testAll(self):
		'''Test basis in check, run, and verify modes;
		also test it when check and verify are both set.
		'''
		log = Logger()

		basisCheck = Basis(args=[Flags.CheckOnly.value])
		basisRun = Basis(customLogger=log)
		basisVerify = Basis(args=[Flags.Verify.value])
		#Also see what happens when both are set.
		basisBoth = Basis(args=[Flags.CheckOnly.value, Flags.Verify.value])

		#Run this in check only, install, and verify modes.
		allModes = ((basisCheck, "Testing check mode..."),
			(basisRun, "Testing run mode..."),
			(basisVerify, "Testing verify mode..."),
			(basisBoth, "Testing verify and check mode (this should act exactly like check mode)..."))
		for modePair in allModes:
			log.info(modePair[1])
			modePair[0].run(_runSteps)

	@classmethod
	def suite(cls):
		'''Returns all tests in the integration test suite.
		'''

		return unittest.TestLoader().loadTestsFromTestCase(IntegrationTest)

if __name__ == "__main__":
	unittest.TextTestRunner(verbosity=2).run(IntegrationTest.suite())



'''Unit tests for packagemanager module.
'''

import unittest
from ....setup import Context
from ....setup import PackageManager, ShellManager
from ....setup import Logger

class PackageManagerTest(unittest.TestCase):
	'''Unit tests for packagemanager module.
	'''

	def testPackageCommands(self):
		'''Tests package management methods.
		'''

		print("")
		logger = Logger()
		ctx = Context(logger)
		if ctx.osIsWindows():
			print("Package test needs non-Windows OS, skipping")
		else:
			package = PackageManager(ShellManager(ctx, logger), logger)
			packageToCheck = "emacs"
			#Don't actually want to modify the system, so just
			#check that the package check doesn't give an exception.
			print("Package check tool: " + package.context.packageCheckInstalledTool)
			package.checkInstalled(packageToCheck)

	#TODO: can we test MSI installation?

if __name__ == "__main__":
	suite = unittest.TestLoader().loadTestsFromTestCase(PackageManagerTest)
	unittest.TextTestRunner(verbosity=2).run(suite)



from .constants import LogLevel, TextCode

#Maps severity levels to color codes.
_logLevelToColorCodeMap = {
	LogLevel.Debug: TextCode.Debug,
	LogLevel.Verbose: TextCode.Verbose,
	LogLevel.Info: TextCode.Reset,
	LogLevel.Warn: TextCode.Warn,
	LogLevel.Error: TextCode.Error,
	LogLevel.Success: TextCode.Ok
}

class RenderMode(object):
	'''Defines rendering settings
	used by Logger._renderedLineForEntry().
	'''

	#Format for rendering strings:
	#	0: time
	#	1: severity prefix
	#	2: message string
	#	3: severity color code

	#Rule format strings.
	#0 is always the parameter of the property
	#to be added, 1 is always the previous string.
	#Strings must be combined in the order listed.
	_kMessageFormatString = "{2}"
	_kLevelTextFormatPrefix = "{1}"
	_kTimeCodeFormatPrefix = "[{0}] "
	_kLevelColorFormatPrefix = "{3}"
	_kLevelColorFormatSuffix = TextCode.Reset

	def __init__(self, showTimecode=True, showLevelText=True, showLevelColor=True):
		'''Initializer for RenderModes.
		By default, timecodes are displayed, as are severity
		levels via both text and code.
		'''

		#If True, timecodes in the format hh:mm:ss:mmmm
		#are added to the rendered entry.
		self.showTimecode = showTimecode
		#If True, text-based severity codes
		#are added to the rendered entry.
		self.showLevelText = showLevelText
		#If True, color highlighting
		#is added to the rendered entry
		#to indicate severity.
		#Currently does nothing.
		self.showLevelColor = showLevelColor

	def formatStringForContext(self):
		'''Returns the format string that this
		render mode indicates should be used.
		'''
		result = RenderMode._kMessageFormatString
		if self.showLevelText:
			result = RenderMode._kLevelTextFormatPrefix + result
		if self.showTimecode:
			result = RenderMode._kTimeCodeFormatPrefix + result
		if self.showLevelColor:
			result = RenderMode._kLevelColorFormatPrefix + result + RenderMode._kLevelColorFormatSuffix

		return result

def renderedLineForEntry(logEntry, formatString):
	'''Returns given log entry as a printable string.
	'''
	prefacesDict = {
		LogLevel.Debug : "(debug) ",
		LogLevel.Verbose : "(verbose) ",
		LogLevel.Info : "",
		LogLevel.Warn : "(warning) ",
		LogLevel.Error : "(error!) ",
		LogLevel.Success : "(success) "
	}
	#Color might not be available, but give it a try.
	logLevel = logEntry[2]
	logTime = logEntry[1]
	logMessage = logEntry[3]
	logColor = _logLevelToColorCodeMap[logLevel]
	return formatString.format(logTime, prefacesDict[logLevel], logMessage, logColor)



'''Defines logger manager class.
'''
from .constants import LogLevel
from .log_entry import LogEntry
from .rendering import RenderMode, renderedLineForEntry

class Logger(object):
	'''Handles event logging and rendering for the application.
	'''

	def __init__(self, minimumPrintableLevel=LogLevel.Info,
	stdOutRenderContext=None, fileRenderContext=None):
		#Entries with a loglevel below this value
		#are not printed.
		self._minimumPrintableLevel = minimumPrintableLevel
		if stdOutRenderContext is None:
			stdOutRenderContext = RenderMode()
		if fileRenderContext is None:
			#Log files are pure text,
			#it doesn't make sense to add color codes
			#to them.
			fileRenderContext = RenderMode(showTimecode=True,
			showLevelText=True,
			showLevelColor=False)
		self._stdOutRenderFormatString = stdOutRenderContext.formatStringForContext()
		self._fileRenderFormatString = fileRenderContext.formatStringForContext()
		#The LogEntries stored by this logger.
		#When dumpEntriesToFile() is called,
		#all values in this list are saved to the
		#given log file.
		self._entries = []

	def _printEntry(self, logEntry):
		lineForEntry = renderedLineForEntry(logEntry, self._stdOutRenderFormatString)
		print(lineForEntry)

	def _addEntry(self, logLevel, message, extraData=None):
		logEntry = LogEntry.entry(logLevel, message, extraData=extraData)
		self._entries.append(logEntry)

		#Print this message to screen if
		#it's above minimum printable level.
		if logLevel >= self._minimumPrintableLevel:
			self._printEntry(logEntry)

	def dumpEntriesToFile(self, filePath):
		'''Saves all entries in the log
		to the specified file, appending to
		the file if it already exists.
		Raises IOError if the file can't be opened.
		'''
		#Just appending, no need to read
		outFile = open(filePath, "a+")
		outFile.writelines("{0}\n".format(renderedLineForEntry(x, \
		self._fileRenderFormatString)) for x in self._entries)

	def entriesOfLevel(self, logLevel):
		'''Gets all log entries exactl matching
		the specified severity level.
		'''
		return [x for x in self._entries if x[2] == logLevel]

	def printLogEntries(self, entryList):
		'''Prints all log entries given.
		These have the same extraData as their original
		entries, but are new entries and go in the result log.
		'''
		for entry in entryList:
			self.logMessage(entry[2], entry[3], entry[5])

	def logMessage(self, logLevel, message, extraData=None):
		'''Logs a message with the given severity to the logger,
		printing it if LogLevel is at `logLevel` or below.
		'''
		assert logLevel >= 0 and logLevel < LogLevel.Count, \
		"Log level should be between 0 and {0}, is actually {1}".format(LogLevel.Count, logLevel)
		self._addEntry(logLevel, message, extraData)

	def debug(self, message, extraData=None):
		'''Logs a debug message to the logger,
		printing it if LogLevel is at Debug or below.
		'''
		self.logMessage(LogLevel.Debug, message, extraData)

	def verbose(self, message, extraData=None):
		'''Logs a verbose message to the logger,
		printing it if LogLevel is at Verbose or below.
		'''
		self.logMessage(LogLevel.Verbose, message, extraData)

	def info(self, message, extraData=None):
		'''Logs an info message to the logger,
		printing it if LogLevel is at Info or below.
		'''
		self.logMessage(LogLevel.Info, message, extraData)

	def warning(self, message, extraData=None):
		'''Logs a warning message to the logger,
		printing it if LogLevel is at Warning or below.
		'''
		self.logMessage(LogLevel.Warn, message, extraData)

	def error(self, message, extraData=None):
		'''Logs an error message to the logger,
		printing it if LogLevel is at Error or below.
		'''
		self.logMessage(LogLevel.Error, message, extraData)

	def success(self, message, extraData=None):
		'''Logs an success message to the logger,
		printing it if LogLevel is at Success or below.
		'''
		self.logMessage(LogLevel.Success, message, extraData)



'''Runs all tests for basis.
'''

import unittest
from .test.integration import IntegrationTest
from .test.unit.all import AllUnitTest

def suite():
	'''Returns all tests for basis.
	'''
	return unittest.TestSuite([AllUnitTest.suite(), IntegrationTest.suite()])

def main():
	unittest.TextTestRunner(verbosity=2).run(suite())

if __name__ == "__main__":
	main()



'''Abstracts common package commands.
'''
import os
from ..logging import Logger
from .exceptions import StepExecutorError, PackageError, ShellError, CommandExecutionError

MSIRebootInitiated = 1641
MSIRebootRequired = 3010
MSINonFailCodes = set([0, MSIRebootRequired, MSIRebootInitiated])
MSICodeDescription = {0 : "Installation was successful",
	MSIRebootRequired : "Installer has requested a reboot",
	MSIRebootInitiated : "Installer is initiating a reboot!"}

class PackageManager(object):
	'''Abstracts common package commands.
	'''

	def __init__(self, shellManager, logger):
		'''Initializer.
		Raises RuntimeError if not provided a ShellManager instance to work with.
		'''
		if not shellManager:
			raise RuntimeError("PackageManager needs ShellManager object to operate.")
		self.context = shellManager.context
		self.shell = shellManager
		self._log = logger
		assert isinstance(self._log, Logger)
		#If True, install/update calls are escalated by default.
		#OSX sets this to False since brew almost never uses root.
		self.escalateShellCalls = True
		if self.context.osIsMacOS():
			self.escalateShellCalls = False

	def install(self, packages, escalate=None):
		'''Installs the given package if possible.
		Raises PackageError if the package could not be installed
		for any reason.
		'''
		if escalate is None:
			escalate = self.escalateShellCalls
		#Auto fail if there's no package installer.
		if not self.context.packageInstallTool:
			raise PackageError("No package installer specified, aborting!")

		self._log.verbose("Installing packages: " + packages, self.context)
		try:
			self.shell.command(self.context.packageInstallTool + " " + packages, escalate)
		except StepExecutorError:
			raise PackageError("Package install failed: " + packages)

	def update(self, escalate=None):
		'''Updates all packages on the system.
		Raises PackageError if the package could not be updated
		for any reason.
		'''
		if escalate is None:
			escalate = self.escalateShellCalls
		#Auto fail if there's no package updater.
		if not self.context.packageUpdateTool:
			raise PackageError("No package updater specified, aborting!")

		self._log.verbose("Updating repository...", self.context)
		try:
			self.shell.command(self.context.packageUpdateTool, escalate)
		except StepExecutorError:
			raise PackageError("Repository update failed!")

	def checkInstalled(self, package, escalate=False):
		'''Checks if a package is installed.
		Returns True if the package is installed,
		False otherwise.
		Raises ShellError if no command is specified for checking packages
		or no packages are given.
		'''

		#Auto fail if there's no package checker.
		if not self.context.packageCheckInstalledTool:
			raise ShellError("No package checker specified, aborting!")
		if not package:
			raise ShellError("No packages marked for installation.")

		self._log.verbose("Checking package: " + package, self.context)
		try:
			self.shell.command(self.context.packageCheckInstalledTool + " " + package, escalate)
		except StepExecutorError:
			self._log.warning("Package not installed: " + package)
			return False
		self._log.verbose("Package is installed: " + package, self.context)
		return True

	def runInstaller(self, installerPath, escalate=None):
		'''Runs an installer file in unattended mode.
		Raises:
			* PackageError if the installer was run but failed installation
			* ShellError if the context doesn't have a command for unattended installation,
			or no installer path is given.
		'''
		if escalate is None:
			escalate = self.escalateShellCalls
		if not self.context.packageRunInstallerTool:
			raise ShellError("Wasn't given an installer \
			command for platform {0}".format(self.context.osIdentifier))
		if not installerPath:
			raise ShellError("No installer specified.")
		try:
			#Normalize the path first!
			installerPath = self.shell.normalizedPath(installerPath)
			self.shell.command(self.context.packageRunInstallerTool.format(installerPath), escalate)
		except CommandExecutionError as e:
			#Is this Windows?
			if self.context.osIsWindows():
				#If so, some error codes aren't failure states.
				if e.exit_code in MSINonFailCodes:
					self._log.info(MSICodeDescription[e.exit_code])
					return
			#Otherwise, bubble up the error!
			raise PackageError("Installer at '{0}' failed: {1}".format(installerPath, e))

	def installPKGFromDMG(self, dmgPath, pkgSubPath):
		'''Attempts to install a PKG from a DMG.
		Valid under macOS only.
		Parameters:
			* dmgPath: Path to the DMG to be mounted.
			* pkgSubPath: The path to the PKG, relative to the DMG.
			If the DMG is at `/example/path.dmg` and
			the PKG is at `/example/path.dmg/actual.pkg`,
			pkgSubPath would be `actual.pkg`.
		Raises:
			* StepExecutorError if the DMG couldn't be mounted,
			if the PKG failed installation for any reason,
			or if the method is not being called on a macOS platform.
		'''
		if not self.context.osIsMacOS():
			raise ShellError("installPKGFromDMG() is only supported under macOS!")

		dmgMount = self.shell.openDMG(dmgPath)
		try:
			#Do the installation.
			installerPath = dmgMount + "/" + pkgSubPath
			self.runInstaller(installerPath)
		finally:
			#Eject the DMG.
			self.shell.closeDMG(dmgMount)

	def installAPPFromDMG(self, dmgPath, appSubPath):
		'''Attempts to copy a .app file from a DMG
		to the system's /Applications folder.
		Valid under macOS only.
		Parameters:
			* dmgPath: Path to the DMG to be mounted.
			* appSubPath: The path to the .app, relative to the DMG.
			If the DMG is at "/example/path.dmg" and
			the .app is at "/example/path.dmg/actual.app",
			appSubPath would be "actual.app".
		Raises:
			* StepExecutorError if the DMG couldn't be mounted,
			if the .app couldn't be copied for any reason,
			or if the method is not being called on a macOS platform.
		'''
		if not self.context.osIsMacOS():
			raise ShellError("installAPPFromDMG() is only supported under macOS!")

		dmgMount = self.shell.openDMG(dmgPath)
		try:
			#Do the installation.
			applicationPath = dmgMount + "/" + appSubPath
			self.shell.command("cp -R {0} /Applications".format(applicationPath))
		finally:
			#Eject the DMG.
			self.shell.closeDMG(dmgMount)



'''Defines constant values used by
modules in the logging package.
'''
from colorama import Fore, Style, init
init()

class LogLevel:
	'''Specifies the severity of a LogEntry.
	Entries below a Logger's minimum severity level
	are still saved to the log file, but are not
	printed to standard output.
	'''
	Debug = 0
	Verbose = 1
	Info = 2
	Warn = 3
	Error = 4
	Success = 5
	Count = 6

class LogEntryType:
	'''Specifies the type of data stored by a given
	LogEntry.
	'''
	GeneralText = 0
	Count = 1

class TextCode(object):
	'''Common format codes used by the log methods.
	'''

	Ok = Fore.GREEN
	Verbose = Fore.MAGENTA
	Debug = Fore.CYAN
	Warn = Fore.YELLOW
	Error = Fore.RED
	Reset = Style.RESET_ALL
	Bold = Style.BRIGHT



class StepResultCategory(object):
	'''Specifies what kind of result a result is.
	'''

	'''The operation could not be completed; during a check,
	indicates that the operation should be performed.
	'''
	Fail = 0
	'''The operation was successfully performed; during a check,
	indicates that the operation does not need to be performed.
	'''
	Success = 1
	'''The operation was not performed at all; during a check,
	indicates that the operation does not need to be performed.
	'''
	Skip = 2
	Count = 3
	Descriptions = ("Step failed",
					"Step succeeded",
					"Step skipped")

class StepResultReason(object):
	'''Specifies why a result occurred.
	'''

	'''Unspecified failure.
	'''
	Fail = 0
	'''Step has not yet been performed.
	'''
	NotDone = 1
	'''The step must always be performed; there is no way to skip this or mark as already done.
	'''
	Mandatory = 2
	'''Something the step relies on does not
	have the right version.
	'''
	WrongVersion = 3
	'''Unspecified success.
	'''
	Success = 4
	'''The step was successfully performed
	and installed new software.
	'''
	Installed = 5
	'''The requirements for the step have already been met.
	'''
	AlreadyDone = 6
	'''The system has the wrong platform for this step,
	and the step is optional.
	'''
	Unnecessary = 7
	Count = 8
	'''Human-readable descriptions for each step.
	'''
	Descriptions = ("Unspecified failure",
						"Not yet performed",
						"Mandatory",
						"Wrong version",
						"Unspecified success",
						"Installed",
						"Already done",
						"Unnecessary")
	'''What type of result this reason belongs to.
	'''
	ToCategory = (StepResultCategory.Fail,
					StepResultCategory.Fail,
					StepResultCategory.Fail,
					StepResultCategory.Fail,
					StepResultCategory.Success,
					StepResultCategory.Success,
					StepResultCategory.Skip,
					StepResultCategory.Skip)

class StepResultObject(object):
	'''Describes the result of an operation on a step.
	'''

	def __init__(self, reason):
		if reason < 0 or reason >= StepResultReason.Count:
			raise RuntimeError("StepResult must have reason in range [0,{0})".format(StepResultReason.Count))
		'''The overall value category of the result.
		'''
		self.category = StepResultReason.ToCategory[reason]
		'''The reason for the result.
		'''
		self.reason = reason
		'''A human-readable description of the result.
		'''
		self.description = StepResultReason.Descriptions[self.reason]

	def __repr__(self):
		return "{0} ({1})".format(StepResultCategory.Descriptions[self.category], self.description)

	def __bool__(self):
		return self.category != StepResultCategory.Fail
	#Python 2 uses __nonzero__ instead.
	__nonzero__ = __bool__

	def __eq__(self, other):
		if isinstance(other, StepResultObject):
			return self.category == other.category and self.reason == other.reason
		return False

	def __ne__(self, other):
		return not self.__eq__(other)

class StepResult(object):
	'''Predefined results.
	'''

	'''Unspecified failure.
	'''
	Fail = StepResultObject(StepResultReason.Fail)
	'''The step has not been performed and must be performed.
	'''
	NotDone = StepResultObject(StepResultReason.NotDone)
	'''The step must always be performed; there is no way to skip this or mark as already done.
	'''
	Mandatory = StepResultObject(StepResultReason.Mandatory)
	'''The system has the wrong version installed.
	'''
	WrongVersion = StepResultObject(StepResultReason.WrongVersion)
	'''Unspecified success.
	'''
	Success = StepResultObject(StepResultReason.Success)
	'''The operation was performed and installed new software.
	'''
	Installed = StepResultObject(StepResultReason.Installed)
	'''The operation was skipped since it already seems to have been performed.
	'''
	AlreadyDone = StepResultObject(StepResultReason.AlreadyDone)
	'''The operation was skipped since it was unnecessary.
	'''
	Unnecessary = StepResultObject(StepResultReason.Unnecessary)

	@classmethod
	def fromBool(cls, value):
		'''Returns the result for each boolean value.
		True maps to Success, False to Fail.
		'''

		if value:
			return cls.Success
		return cls.Fail
